__version__ = '0.8.8'
__url__ = 'https://github.com/schnittstabil/findd'
__bugs_url__ = __url__ + '/issues'
__author__ = 'Michael Mayer'
__author_email__ = 'michael@schnittstabil.de'
__license__ = 'MIT'
